@@ -9,6 +9,7 @@
 
 #include <OpenImageIO/color.h>
 #include <OpenImageIO/dassert.h>
+#include <OpenImageIO/filter.h>
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/hash.h>
 #include <OpenImageIO/imageio.h>
@@ -20,6 +21,7 @@
 #include <OpenImageIO/timer.h>
 #include <OpenImageIO/typedesc.h>
 
+#include "buildopts.h"
 #include "imageio_pvt.h"
 
 OIIO_NAMESPACE_BEGIN
@@ -46,12 +48,15 @@ atomic_int oiio_try_all_readers(1);
 #endif
 // Should we use "Exr core C library"?
 int openexr_core(OIIO_OPENEXR_CORE_DEFAULT);
+int jpeg_com_attributes(1);
+int png_linear_premult(0);
 int tiff_half(0);
 int tiff_multithread(1);
 int dds_bc5normal(0);
 int limit_channels(1024);
 int limit_imagesize_MB(std::min(32 * 1024,
                                 int(Sysutil::physical_memory() >> 20)));
+int imageinput_strict(0);
 ustring font_searchpath(Sysutil::getenv("OPENIMAGEIO_FONTS"));
 ustring plugin_searchpath(OIIO_DEFAULT_PLUGIN_SEARCHPATH);
 std::string format_list;         // comma-separated list of all formats
@@ -60,7 +65,6 @@ std::string output_format_list;  // comma-separated list of writable formats
 std::string extension_list;      // list of all extensions for all formats
 std::string library_list;        // list of all libraries for all formats
 int oiio_log_times = Strutil::stoi(Sysutil::getenv("OPENIMAGEIO_LOG_TIMES"));
-int oiio_print_uncaught_errors(1);
 std::vector<float> oiio_missingcolor;
 }  // namespace pvt
 
@@ -69,7 +73,7 @@ using namespace pvt;
 
 namespace {
 // Hidden global OIIO data.
-static spin_mutex attrib_mutex;
+static std::recursive_mutex attrib_mutex;
 static const int maxthreads = 512;  // reasonable maximum for sanity check
 
 class TimingLog {
@@ -114,10 +118,10 @@ public:
             double time         = item.second.first;
             double percall      = time / ncalls;
             bool use_ms_percall = (percall < 0.1);
-            out << Strutil::sprintf("%-25s%6d %7.3fs  (avg %6.2f%s)\n",
-                                    item.first, ncalls, time,
-                                    percall * (use_ms_percall ? 1000.0 : 1.0),
-                                    use_ms_percall ? "ms" : "s");
+            OIIO::print(out, "{:25s}{:6d} {:7.3f}s  (avg {:6.2f}{})\n",
+                        item.first, ncalls, time,
+                        percall * (use_ms_percall ? 1000.0 : 1.0),
+                        use_ms_percall ? "ms" : "s");
         }
         return out.str();
     }
@@ -167,7 +171,6 @@ hw_simd_caps()
     if (cpu_has_avx512dq())    caps.emplace_back ("avx512dq");
     if (cpu_has_avx512ifma())  caps.emplace_back ("avx512ifma");
     if (cpu_has_avx512pf())    caps.emplace_back ("avx512pf");
-    if (cpu_has_avx512er())    caps.emplace_back ("avx512er");
     if (cpu_has_avx512cd())    caps.emplace_back ("avx512cd");
     if (cpu_has_avx512bw())    caps.emplace_back ("avx512bw");
     if (cpu_has_avx512vl())    caps.emplace_back ("avx512vl");
@@ -199,7 +202,6 @@ oiio_simd_caps()
     if (OIIO_AVX512DQ_ENABLED)   caps.emplace_back ("avx512dq");
     if (OIIO_AVX512IFMA_ENABLED) caps.emplace_back ("avx512ifma");
     if (OIIO_AVX512PF_ENABLED)   caps.emplace_back ("avx512pf");
-    if (OIIO_AVX512ER_ENABLED)   caps.emplace_back ("avx512er");
     if (OIIO_AVX512CD_ENABLED)   caps.emplace_back ("avx512cd");
     if (OIIO_AVX512BW_ENABLED)   caps.emplace_back ("avx512bw");
     if (OIIO_AVX512VL_ENABLED)   caps.emplace_back ("avx512vl");
@@ -211,6 +213,64 @@ oiio_simd_caps()
     // clang-format on
 }
 
+
+static std::string
+oiio_build_compiler()
+{
+    using Strutil::fmt::format;
+
+    std::string comp;
+#if OIIO_INTEL_CLASSIC_COMPILER_VERSION
+    comp = format("Intel icc {}", OIIO_INTEL_CLASSIC_COMPILER_VERSION);
+#elif OIIO_INTEL_LLVM_COMPILER
+    comp = format("Intel icx {}.{}", __clang_major__, __clang_minor__);
+#elif OIIO_APPLE_CLANG_VERSION
+    comp = format("Apple clang {}.{}", __clang_major__, __clang_minor__);
+#elif OIIO_CLANG_VERSION
+    comp = format("clang {}.{}", __clang_major__, __clang_minor__);
+#elif OIIO_GNUC_VERSION
+    comp = format("gcc {}.{}", __GNUC__, __GNUC_MINOR__);
+#elif OIIO_MSVS_VERSION
+    comp = format("MSVS {}", OIIO_MSVS_VERSION);
+#else
+    comp = "unknown compiler?";
+#endif
+    return comp;
+}
+
+
+static std::string
+oiio_build_platform()
+{
+    std::string platform;
+#if defined(__linux__)
+    platform = "Linux";
+#elif defined(__APPLE__)
+    platform = "MacOS";
+#elif defined(_WIN32)
+    platform = "Windows";
+#elif defined(__MINGW32__)
+    platform = "MinGW";
+#elif defined(__FreeBSD__)
+    platform = "FreeBSD";
+#else
+    platform = "UnknownOS";
+#endif
+    platform += "/";
+#if defined(__x86_64__)
+    platform += "x86_64";
+#elif defined(__i386__)
+    platform += "i386";
+#elif defined(_M_ARM64) || defined(__aarch64__) || defined(__aarch64)
+    platform += "ARM";
+#else
+    platform = "unknown arch?";
+#endif
+    return platform;
+}
+
+
+
 void
 shutdown()
 {
@@ -226,51 +286,10 @@ openimageio_version()
 
 
 
-// ErrorHolder houses a string, with the addition that when it is destroyed,
-// it will disgorge any un-retrieved error messages, in an effort to help
-// beginning users diagnose their problems if they have forgotten to call
-// geterror().
-struct ErrorHolder {
-    std::string error_msg;
-
-    ~ErrorHolder()
-    {
-        if (!error_msg.empty() && pvt::oiio_print_uncaught_errors) {
-            OIIO::print(
-                "OpenImageIO exited with a pending error message that was never\n"
-                "retrieved via OIIO::geterror(). This was the error message:\n{}\n",
-                error_msg);
-        }
-    }
-};
-
-
-
-// To avoid thread oddities, we have the storage area buffering error
-// messages for append_error()/geterror() be thread-specific.
-static thread_local ErrorHolder error_msg_holder;
-
-
 void
 pvt::append_error(string_view message)
 {
-    // Remove a single trailing newline
-    if (message.size() && message.back() == '\n')
-        message.remove_suffix(1);
-    std::string& error_msg(error_msg_holder.error_msg);
-    OIIO_ASSERT(
-        error_msg.size() < 1024 * 1024 * 16
-        && "Accumulated error messages > 16MB. Try checking return codes!");
-    // If we are appending to existing error messages, separate them with
-    // a single newline.
-    if (error_msg.size() && error_msg.back() != '\n')
-        error_msg += '\n';
-    error_msg += std::string(message);
-
-    // Remove a single trailing newline
-    if (message.size() && message.back() == '\n')
-        message.remove_suffix(1);
-    error_msg = std::string(message);
+    Strutil::pvt::append_error(message);
 }
 
 
@@ -278,8 +297,7 @@ pvt::append_error(string_view message)
 bool
 has_error()
 {
-    std::string& error_msg(error_msg_holder.error_msg);
-    return !error_msg.empty();
+    return Strutil::pvt::has_error();
 }
 
 
@@ -287,11 +305,7 @@ has_error()
 std::string
 geterror(bool clear)
 {
-    std::string& error_msg(error_msg_holder.error_msg);
-    std::string e = error_msg;
-    if (clear)
-        error_msg.clear();
-    return e;
+    return Strutil::pvt::geterror(clear);
 }
 
 
@@ -305,7 +319,7 @@ debug(string_view message)
 
 
 void
-pvt::log_time(string_view key, const Timer& timer, int count)
+log_time(string_view key, const Timer& timer, int count)
 {
     timing_log(key, timer, count);
 }
@@ -327,7 +341,13 @@ attribute(string_view name, TypeDesc type, const void* val)
         default_thread_pool()->resize(ot - 1);
         return true;
     }
-    spin_lock lock(attrib_mutex);
+    if (Strutil::starts_with(name, "gpu:")
+        || Strutil::starts_with(name, "cuda:")) {
+        return pvt::gpu_attribute(name, type, val);
+    }
+
+    // Things below here need to buarded by the attrib_mutex
+    std::lock_guard lock(attrib_mutex);
     if (name == "read_chunk" && type == TypeInt) {
         oiio_read_chunk = *(const int*)val;
         return true;
@@ -348,6 +368,14 @@ attribute(string_view name, TypeDesc type, const void* val)
         openexr_core = *(const int*)val;
         return true;
     }
+    if (name == "jpeg:com_attributes" && type == TypeInt) {
+        jpeg_com_attributes = *(const int*)val;
+        return true;
+    }
+    if (name == "png:linear_premult" && type == TypeInt) {
+        png_linear_premult = *(const int*)val;
+        return true;
+    }
     if (name == "tiff:half" && type == TypeInt) {
         tiff_half = *(const int*)val;
         return true;
@@ -380,6 +408,10 @@ attribute(string_view name, TypeDesc type, const void* val)
         imagebuf_use_imagecache = *(const int*)val;
         return true;
     }
+    if (name == "imageinput:strict" && type == TypeInt) {
+        imageinput_strict = *(const int*)val;
+        return true;
+    }
     if (name == "use_tbb" && type == TypeInt) {
         oiio_use_tbb = *(const int*)val;
         return true;
@@ -417,6 +449,7 @@ attribute(string_view name, TypeDesc type, const void* val)
 bool
 getattribute(string_view name, TypeDesc type, void* val)
 {
+    using Strutil::fmt::format;
     if (name == "threads" && type == TypeInt) {
         *(int*)val = oiio_threads;
         return true;
@@ -425,7 +458,13 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(ustring*)val = ustring(OIIO_VERSION_STRING);
         return true;
     }
-    spin_lock lock(attrib_mutex);
+    if (Strutil::starts_with(name, "gpu:")
+        || Strutil::starts_with(name, "cuda:")) {
+        return pvt::gpu_getattribute(name, type, val);
+    }
+
+    // Things below here need to buarded by the attrib_mutex
+    std::lock_guard lock(attrib_mutex);
     if (name == "read_chunk" && type == TypeInt) {
         *(int*)val = oiio_read_chunk;
         return true;
@@ -480,6 +519,30 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(ustring*)val = ustring(Strutil::join(font_list(), ";"));
         return true;
     }
+    if (name == "font_family_list" && type == TypeString) {
+        *(ustring*)val = ustring(Strutil::join(font_family_list(), ";"));
+        return true;
+    }
+    if (Strutil::starts_with(name, "font_style_list:") && type == TypeString) {
+        string_view family = name.substr(strlen("font_style_list:"));
+        *(ustring*)val = ustring(Strutil::join(font_style_list(family), ";"));
+        return true;
+    }
+    if (Strutil::starts_with(name, "font_filename:") && type == TypeString) {
+        std::vector<string_view> tokens;
+        Strutil::split(name, tokens, ":");
+        string_view family = tokens.size() >= 1 ? tokens[1] : string_view();
+        string_view style  = tokens.size() >= 2 ? tokens[2] : string_view();
+        *(ustring*)val     = ustring(font_filename(family, style));
+        return true;
+    }
+    if (name == "filter_list" && type == TypeString) {
+        std::vector<string_view> filternames;
+        for (int i = 0, e = Filter2D::num_filters(); i < e; ++i)
+            filternames.emplace_back(Filter2D::get_filterdesc(i).name);
+        *(ustring*)val = ustring(Strutil::join(filternames, ";"));
+        return true;
+    }
     if (name == "exr_threads" && type == TypeInt) {
         *(int*)val = oiio_exr_threads;
         return true;
@@ -488,6 +551,14 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(int*)val = openexr_core;
         return true;
     }
+    if (name == "jpeg:com_attributes" && type == TypeInt) {
+        *(int*)val = jpeg_com_attributes;
+        return true;
+    }
+    if (name == "png:linear_premult" && type == TypeInt) {
+        *(int*)val = png_linear_premult;
+        return true;
+    }
     if (name == "tiff:half" && type == TypeInt) {
         *(int*)val = tiff_half;
         return true;
@@ -520,6 +591,10 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(int*)val = imagebuf_use_imagecache;
         return true;
     }
+    if (name == "imageinput:strict" && type == TypeInt) {
+        *(int*)val = imageinput_strict;
+        return true;
+    }
     if (name == "use_tbb" && type == TypeInt) {
         *(int*)val = oiio_use_tbb;
         return true;
@@ -540,14 +615,30 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(ustring*)val = ustring(hw_simd_caps());
         return true;
     }
-    if (name == "oiio:simd" && type == TypeString) {
+    if ((name == "build:simd" || name == "oiio:simd") && type == TypeString) {
         *(ustring*)val = ustring(oiio_simd_caps());
         return true;
     }
+    if (name == "build:compiler" && type == TypeString) {
+        *(ustring*)val = ustring(oiio_build_compiler());
+        return true;
+    }
+    if (name == "build:platform" && type == TypeString) {
+        *(ustring*)val = ustring(oiio_build_platform());
+        return true;
+    }
+    if (name == "build:dependencies" && type == TypeString) {
+        *(ustring*)val = ustring(OIIO_ALL_BUILD_DEPS_FOUND);
+        return true;
+    }
     if (name == "resident_memory_used_MB" && type == TypeInt) {
         *(int*)val = int(Sysutil::memory_used(true) >> 20);
         return true;
     }
+    if (name == "resident_memory_used_MB" && type == TypeFloat) {
+        *(float*)val = float(Sysutil::memory_used(true) >> 20);
+        return true;
+    }
     if (name == "missingcolor" && type.basetype == TypeDesc::FLOAT
         && oiio_missingcolor.size()) {
         // missingcolor as float array
@@ -574,10 +665,6 @@ getattribute(string_view name, TypeDesc type, void* val)
                                             (v >> 16) & 0xff, (v >> 8) & 0xff);
         return true;
     }
-    if (name == "opencv_version" && type == TypeInt) {
-        *(int*)val = OIIO::pvt::opencv_version;
-        return true;
-    }
     if (name == "IB_local_mem_current" && type == TypeInt64) {
         *(long long*)val = IB_local_mem_current;
         return true;
@@ -983,38 +1070,9 @@ add_dither(int nchannels, int width, int height, int depth, float* data,
            unsigned int ditherseed, int chorigin, int xorigin, int yorigin,
            int zorigin)
 {
-#if OIIO_VERSION < OIIO_MAKE_VERSION(2, 4, 0)
-    // Old: uniform random noise
-    ImageSpec::auto_stride(xstride, ystride, zstride, sizeof(float), nchannels,
-                           width, height);
-    char* plane = (char*)data;
-    for (int z = 0; z < depth; ++z, plane += zstride) {
-        char* scanline = plane;
-        for (int y = 0; y < height; ++y, scanline += ystride) {
-            char* pixel = scanline;
-            uint32_t ba = (z + zorigin) * 1311 + yorigin + y;
-            uint32_t bb = ditherseed + (chorigin << 24);
-            uint32_t bc = xorigin;
-            for (int x = 0; x < width; ++x, pixel += xstride) {
-                float* val = (float*)pixel;
-                for (int c = 0; c < nchannels; ++c, ++val, ++bc) {
-                    bjhash::bjmix(ba, bb, bc);
-                    int channel = c + chorigin;
-                    if (channel == alpha_channel || channel == z_channel)
-                        continue;
-                    float dither
-                        = bc / float(std::numeric_limits<uint32_t>::max());
-                    *val += ditheramplitude * (dither - 0.5f);
-                }
-            }
-        }
-    }
-#else
-    // New: Use blue noise for our dither
     add_bluenoise(nchannels, width, height, depth, data, xstride, ystride,
                   zstride, ditheramplitude, alpha_channel, z_channel,
                   ditherseed, chorigin, xorigin, yorigin, zorigin);
-#endif
 }
 
 
